"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.load = void 0;
const typedoc_1 = require("typedoc");
// store function symbol -> reflection id that documents it
const storeLinks = new Map();
// properties/variables which reference a store
const storeReferences = new Map();
function load(app) {
    app.converter.on(typedoc_1.Converter.EVENT_CREATE_DECLARATION, handleDefineComponent);
    app.converter.on(typedoc_1.Converter.EVENT_CREATE_DECLARATION, collectStoreReferences);
    app.converter.on(typedoc_1.Converter.EVENT_CREATE_SIGNATURE, handleDefineStore);
    app.converter.on(typedoc_1.Converter.EVENT_END, (context) => {
        for (const [src, target] of storeLinks) {
            const references = storeReferences.get(src) || [];
            for (const src of references) {
                src.type = typedoc_1.ReferenceType.createBrokenReference("PiniaStore", context.project);
                src.type.typeArguments = [
                    typedoc_1.ReferenceType.createResolvedReference(target.name, target, context.project),
                ];
                delete src.defaultValue;
            }
        }
        storeLinks.clear();
        storeReferences.clear();
    });
}
exports.load = load;
function collectStoreReferences(context, refl) {
    if (!refl.kindOf(typedoc_1.ReflectionKind.Property)) {
        return;
    }
    const declaration = refl.project
        .getSymbolFromReflection(refl)
        ?.getDeclarations()?.[0];
    if (!declaration ||
        !typedoc_1.TypeScript.isPropertyAssignment(declaration) ||
        !declaration.initializer ||
        !typedoc_1.TypeScript.isCallExpression(declaration.initializer)) {
        return;
    }
    let fnName = declaration.initializer.expression;
    while (typedoc_1.TypeScript.isBinaryExpression(fnName)) {
        fnName = fnName.right;
    }
    if (!typedoc_1.TypeScript.isIdentifier(fnName))
        return;
    const symbol = context.getSymbolAtLocation(fnName);
    if (!symbol)
        return;
    if (storeReferences.has(symbol)) {
        storeReferences.get(symbol).push(refl);
    }
    else {
        storeReferences.set(symbol, [refl]);
    }
}
function handleDefineComponent(context, refl) {
    if (!refl.kindOf(typedoc_1.ReflectionKind.Variable) ||
        refl.type?.type !== "reference" ||
        refl.type.package !== "@vue/runtime-core" ||
        refl.type.qualifiedName !== "DefineComponent") {
        return;
    }
    const declaration = refl.project
        .getSymbolFromReflection(refl)
        ?.getDeclarations()?.[0];
    if (!declaration)
        return;
    const type = context.getTypeAtLocation(declaration);
    if (!type)
        return;
    const instanceType = type.getConstructSignatures()[0]?.getReturnType();
    if (!instanceType)
        return;
    refl.setFlag(typedoc_1.ReflectionFlag.Const, false);
    refl.kind = typedoc_1.ReflectionKind.Class;
    for (const prop of instanceType.getProperties()) {
        context.converter.convertSymbol(context.withScope(refl), prop);
    }
}
const piniaProperties = new Set([
    "$state",
    "$patch",
    "$reset",
    "$subscribe",
    "$onAction",
    "$dispose",
    "$id",
    "_customProperties",
]);
function handleDefineStore(context, refl) {
    if (!refl.kindOf(typedoc_1.ReflectionKind.CallSignature) ||
        refl?.type?.type !== "reference" ||
        refl.type.package !== "pinia" ||
        refl.type.qualifiedName !== "Store") {
        return;
    }
    const symbol = context.project.getSymbolFromReflection(refl.parent);
    if (!symbol)
        return;
    const type = context.checker
        .getTypeOfSymbol(symbol)
        .getCallSignatures()[0]
        ?.getReturnType();
    if (!type)
        return;
    storeLinks.set(symbol, refl.parent);
    // Kind of hacky... TypeDoc converts callable things as functions unless they're
    // on a class or interface, which we want to kind of pretend this store is.
    const originalKind = refl.parent.kind;
    refl.parent.kind = typedoc_1.ReflectionKind.Class;
    for (const prop of type.getProperties()) {
        if (!piniaProperties.has(prop.name)) {
            context.converter.convertSymbol(context.withScope(refl.parent), prop);
        }
    }
    refl.parent.kind = originalKind;
    refl.type = typedoc_1.ReferenceType.createBrokenReference("PiniaStore", context.project);
    refl.type.typeArguments = [
        typedoc_1.ReferenceType.createResolvedReference(refl.parent.name, refl.parent, context.project),
    ];
}
//# sourceMappingURL=plugin.js.map